import subprocess
import urllib
import codecs
import time
import re
import os

from django.conf import settings

UPDATE_INTERVAL = 60 * 60 * 24
URL = "http://udd.debian.org/udd.sql.gz"
LOCAL_FILE = os.path.join(settings.TEMP_PATH, "udd.sql.gz")
UNZIPPED_FILE = os.path.join(settings.TEMP_PATH, "udd.sql")
UPDATED_FILE = os.path.join(settings.TEMP_PATH, "new-udd.sql")

def needs_update(data_file):
    return (not os.path.exists(data_file)) or \
           (time.time()-os.path.getctime(data_file) >= UPDATE_INTERVAL)

def extract_good_parts():
    new_script = []
    stuff = [ "CREATE EXTENSION IF NOT EXISTS debversion WITH SCHEMA public;",
              "CREATE TABLE ubuntu_upload_history (",
              "COPY ubuntu_upload_history (",
              "ALTER TABLE ONLY ubuntu_upload_history",
              "REVOKE ALL ON TABLE ubuntu_upload_history FROM PUBLIC;",
              "REVOKE ALL ON TABLE ubuntu_upload_history FROM udd;",
              "GRANT ALL ON TABLE ubuntu_upload_history TO udd;",
              "GRANT SELECT ON TABLE ubuntu_upload_history TO PUBLIC;",
            ]
    copy = False
    with codecs.open(UNZIPPED_FILE, mode="r", encoding="utf-8", errors="replace") as f:
        for line in f:
            if filter(lambda a: line.startswith(a), stuff):
                copy = True
            if not line.strip():
                copy = False
            if copy:
                new_line = re.sub("\\n", "\n", unicode(line.strip()))
                new_script += [ new_line ]
    new_script += [""]
    if os.path.exists(UPDATED_FILE):
        os.remove(UPDATED_FILE)
    with codecs.open(UPDATED_FILE, mode="w", encoding="utf-8", errors="replace") as f:
        f.write(u"\n".join(new_script))


def get_new_udd_data():
    if needs_update(UNZIPPED_FILE):
        try:
            sock = urllib.urlopen(URL)
        except:
            return None
        sock.close()
        if os.path.exists(LOCAL_FILE):
            os.remove(LOCAL_FILE)
        urllib.urlretrieve(URL, LOCAL_FILE)
        if os.path.exists(UNZIPPED_FILE):
            os.remove(UNZIPPED_FILE)
        subprocess.call(["gunzip", LOCAL_FILE])
        extract_good_parts()
    new_script = UPDATED_FILE
    return new_script

