from uploads.models import Uploads, People
from django.core.management.base import NoArgsCommand
from datetime import timedelta
from django.utils import timezone
from uploads.common.launchpad import lp_login as lp

class Command(NoArgsCommand):
    help = "Create entry in people table if new LP id is found in uploads table."

    def import_people(self):
        lpids = Uploads.objects.values_list('lpid_changer', flat=True).distinct()
        for lpid in lpids:
            ul = Uploads.objects.filter(lpid_changer=lpid).order_by('timestamp')[0]
            last_ul = Uploads.objects.filter(lpid_changer=lpid).order_by('timestamp').reverse()[0]
            obj, created = People.objects.get_or_create(lpid=lpid,
                                                        defaults={
                                                        'name':ul.name_changer,
                                                        'email':ul.email_changer,
                                                        'first_upload':ul,
                                                        'last_upload':last_ul})

    def check_is_active(self):
        lpids = People.objects.values_list('lpid', flat=True).distinct()
        for lp in lpids:
            last_ul = Uploads.objects.filter(lpid_changer=lp).order_by('timestamp').reverse()[0].timestamp
            cutoff_date = timezone.now()-timedelta(days=4*30)  # 4 months
            person = People.objects.get(lpid=lp)
            if last_ul < cutoff_date:
                if person.is_active is not False:
                    person.is_active = False
                    person.save()
            else:
                if person.is_active is not True:
                    person.is_active = True
                    person.save()

    def total_uploads(self):
        for p in People.objects.values_list('lpid', flat=True).distinct():
            person = People.objects.get(lpid=p)
            all_uploads = Uploads.objects.filter(lpid_changer=p)
            total_uploads = len(all_uploads)
            if person.total_uploads != total_uploads:
                person.total_uploads = total_uploads
                person.save()

    def last_seen(self):
        for p in People.objects.values_list('lpid', flat=True).distinct():
            person = People.objects.get(lpid=p)
            last_ul = Uploads.objects.filter(lpid_changer=p).order_by('timestamp').reverse()[0]
            if person.last_upload != last_ul:
                person.last_upload = last_ul
                person.save()

    def is_ubuntu_dev(self):
        launchpad = lp('d-a-t', anonymous=True, lp_service='production')
        ubuntu_devs = [a.name for a in launchpad.people['ubuntu-dev'].participants]
        for p in People.objects.all():
            if p.lpid in ubuntu_devs and p.ubuntu_dev is not True:
                p.ubuntu_dev = True
                p.save()
            elif p.lpid not in ubuntu_devs and p.ubuntu_dev is not False:
                p.ubuntu_dev = False
                p.save()

    def handle_noargs(self, **options):
        self.import_people()
        self.check_is_active()
        self.total_uploads()
        self.last_seen()
        self.is_ubuntu_dev()
