from django.shortcuts import render, get_object_or_404
from django.http import HttpResponseRedirect
from collections import defaultdict
from datetime import timedelta
from django.utils import timezone
from django.contrib import messages

from uploads.models import Uploads, People
from uploads.forms import NotesForm, BooleanForm


def first_timers(request):
    output = []
    for p in People.objects.all():
        if p.total_uploads < 5:
            color = "lt5"
        if p.total_uploads >= 5 and p.total_uploads < 10:
            color = "gt5"
        if p.total_uploads >= 10 and p.total_uploads < 20:
            color = "gt10"
        if p.total_uploads >= 20 and p.total_uploads < 50:
            color = "gt20"
        if p.total_uploads >= 50:
            color = "gt50"
        ul_info = {'name': p.name, 'lpid': p.lpid,
                   'total_uploads': p.total_uploads,
                   'package': p.first_upload.package,
                   'release': p.first_upload.release,
                   'first_seen': p.first_upload.timestamp.date(),
                   'is_active': p.is_active,
                   'last_seen': p.last_upload.timestamp.date(),
                   'color': color}
        output.append(ul_info)
    return render(request, 'first_timers.html', {'output': output})


def person_detail(request, lpid):
    person = get_object_or_404(People, lpid=lpid)
    uploads = Uploads.objects.filter(lpid_changer=lpid)
    recent_uploads = uploads.order_by('timestamp').reverse()[0:10]
    packages = []
    ppu_candidates = []
    for ul in uploads:
        packages += [ul.package]
    appearances = defaultdict(int)
    for curr in packages:
        appearances[curr] += 1
    for pkg in appearances:
        if appearances[pkg] > 5:
            ppu_candidates += [pkg]
    if request.method == 'POST':
        if 'save_notes' in request.POST:
            notes_form = NotesForm(request.POST)
            contacted = BooleanForm(initial={'checkbox': person.contacted})
            if notes_form.is_valid():
                person.notes = notes_form.cleaned_data['notes']
                person.save()
                messages.success(request, 'Change successfully saved...')
                return HttpResponseRedirect('#')
        elif 'update_contacted' in request.POST:
            contacted = BooleanForm(request.POST)
            notes_form = NotesForm(initial={'notes': person.notes})
            if contacted.is_valid():
                print contacted.cleaned_data['checkbox']
                person.contacted = contacted.cleaned_data['checkbox']
                person.save()
                messages.success(request, 'Change successfully saved...')
                return HttpResponseRedirect('#')
    else:
        notes_form = NotesForm(initial={'notes': person.notes})
        contacted = BooleanForm(initial={'checkbox': person.contacted})
    return render(request, 'person.html', {'person': person,
                                           'recent_uploads': recent_uploads,
                                           'ppu_candidates': ppu_candidates,
                                           'notes_form': notes_form,
                                           'contacted': contacted})


def recent_contributors(request):
    recent_contributors = []
    for p in People.objects.all():
        cutoff_date = timezone.now() - timedelta(days=2 * 30)
        if p.last_upload.timestamp > cutoff_date:
            recent_contributors += [p]
    return render(request, 'recent_contributors.html',
                  {'recent_contributors': recent_contributors})


def lost_contributors(request):
    lost_contributors = []
    for p in People.objects.all():
        one_year = timezone.now() - timedelta(days=365)
        two_months = timezone.now() - timedelta(days=2 * 30)
        ul_date = p.last_upload.timestamp
        if  ul_date > one_year and ul_date < two_months:
            lost_contributors += [p]
    return render(request, 'lost_contributors.html',
                  {'lost_contributors': lost_contributors})


def potential_devs(request):
    potential_devs = []
    for p in People.objects.all():
        six_months = timezone.now() - timedelta(days=6 * 30)
        if ((p.first_upload.timestamp < six_months
             and p.ubuntu_dev is not True
             and p.total_uploads >= 40
             and p.is_active is True)):
                potential_devs += [p]
    return render(request, 'potential_devs.html',
                  {'potential_devs': potential_devs})
